
#define CINTERFACE

#include <windows.h>  
#include "write.h"  
#include "commctrl.h"


#if defined (WIN32)
	#define IS_WIN32 TRUE
#else
	#define IS_WIN32 FALSE
#endif

#define IS_NT      IS_WIN32 && (BOOL)(GetVersion() < 0x80000000)
#define IS_WIN32S  IS_WIN32 && (BOOL)(!(IS_NT) && (LOBYTE(LOWORD(GetVersion()))<4))
#define IS_WIN95   (BOOL)(!(IS_NT) && !(IS_WIN32S)) && IS_WIN32

HINSTANCE hInst;   // current instance

LPCTSTR lpszAppName = "MyApp";
LPCTSTR lpszTitle   = "ImageList_Write()"; 


BOOL RegisterWin95( CONST WNDCLASS* lpwc );


int APIENTRY WinMain( HINSTANCE hInstance, HINSTANCE hPrevInstance,
                      LPTSTR lpCmdLine, int nCmdShow)
{
   MSG      msg;
   HWND     hWnd; 
   WNDCLASS wc;

   wc.style         = CS_HREDRAW | CS_VREDRAW;
   wc.lpfnWndProc   = (WNDPROC)WndProc;       
   wc.cbClsExtra    = 0;                      
   wc.cbWndExtra    = 0;                      
   wc.hInstance     = hInstance;              
   wc.hIcon         = LoadIcon (hInstance, lpszAppName); 
   wc.hCursor       = LoadCursor(NULL, IDC_ARROW);
   wc.hbrBackground = (HBRUSH)(COLOR_WINDOW+1);
   wc.lpszMenuName  = lpszAppName;              
   wc.lpszClassName = lpszAppName;              

   if ( IS_WIN95 )
   {
      if ( !RegisterWin95( &wc ) )
         return( FALSE );
   }
   else if ( !RegisterClass( &wc ) )
      return( FALSE );

   hInst = hInstance; 

   hWnd = CreateWindow( lpszAppName, 
                        lpszTitle,    
                        WS_OVERLAPPEDWINDOW, 
                        CW_USEDEFAULT, 0, 
                        CW_USEDEFAULT, 0,  
                        NULL,              
                        NULL,              
                        hInstance,         
                        NULL               
                      );

   if ( !hWnd ) 
      return( FALSE );

   ShowWindow( hWnd, nCmdShow ); 
   UpdateWindow( hWnd );         

   while( GetMessage( &msg, NULL, 0, 0) )   
   {
      TranslateMessage( &msg ); 
      DispatchMessage( &msg );  
   }

   return( msg.wParam ); 
}


BOOL RegisterWin95( CONST WNDCLASS* lpwc )
{
   WNDCLASSEX wcex;

   wcex.style         = lpwc->style;
   wcex.lpfnWndProc   = lpwc->lpfnWndProc;
   wcex.cbClsExtra    = lpwc->cbClsExtra;
   wcex.cbWndExtra    = lpwc->cbWndExtra;
   wcex.hInstance     = lpwc->hInstance;
   wcex.hIcon         = lpwc->hIcon;
   wcex.hCursor       = lpwc->hCursor;
   wcex.hbrBackground = lpwc->hbrBackground;
   wcex.lpszMenuName  = lpwc->lpszMenuName;
   wcex.lpszClassName = lpwc->lpszClassName;

   // Added elements for Windows 95.
   //...............................
   wcex.cbSize = sizeof(WNDCLASSEX);
   wcex.hIconSm = LoadImage(wcex.hInstance, lpwc->lpszClassName, 
                            IMAGE_ICON, 16, 16,
                            LR_DEFAULTCOLOR );
			
   return RegisterClassEx( &wcex );
}


LRESULT CALLBACK WndProc( HWND hWnd, UINT uMsg, WPARAM wParam, LPARAM lParam )
{
static HIMAGELIST hList  = NULL;

   switch( uMsg )
   {
      case WM_CREATE :
              InitCommonControls();

              // Create the image list.
              //.......................
              hList = ImageList_LoadBitmap( hInst, "BMP", 32, 1, RGB( 255, 0, 0 ) );
              break;

      case WM_PAINT :
              {
                 PAINTSTRUCT ps;

                 // Paint the images on the client area.
                 //.....................................
                 BeginPaint( hWnd, &ps );
                 if ( hList )
                 {
                    ImageList_Draw( hList, 0, ps.hdc, 10, 10, ILD_NORMAL );
                    ImageList_Draw( hList, 1, ps.hdc, 60, 10, ILD_NORMAL );
                 }
                 EndPaint( hWnd, &ps );
              }
              break;

      case WM_COMMAND :
              switch( LOWORD( wParam ) )
              {
                 case IDM_SAVE :
                        if ( hList )
                        {
                           LPSTORAGE lpst;
                           LPSTREAM  lpstr;

                           // Create the Compound document.
                           //..............................
                           if ( StgCreateDocfile( OLESTR("Image List"), 
                                                  STGM_READWRITE | STGM_SHARE_EXCLUSIVE | STGM_CREATE, 
                                                  0, &lpst ) == S_OK )
                           {
                              // Create a stream within the document.
                              //.....................................
                              lpst->lpVtbl->CreateStream( lpst, OLESTR("Images"), 
                                                  STGM_READWRITE | STGM_SHARE_EXCLUSIVE | STGM_CREATE,
						                                0, 0, &lpstr );

                              // Save the image list in the document.
                              //.....................................
                              ImageList_Write( hList, lpstr );
                              ImageList_Destroy( hList );
                              hList = NULL;

                              InvalidateRect( hWnd, NULL, TRUE );
                              UpdateWindow( hWnd );

                              // Release the IStream and IStorage objects.
                              //..........................................
                              lpstr->lpVtbl->Release(lpstr);
                              lpst->lpVtbl->Release(lpst);

                              MessageBox( hWnd, "The image list is saved.", 
                                          lpszTitle, MB_OK | MB_ICONINFORMATION );
                           }
                        }
                        break;

                 case IDM_OPEN :
                        if ( !hList )
                        {
                           LPSTORAGE lpst;
                           LPSTREAM  lpstr;

                           // Open the compound document.
                           //............................
                           if ( StgOpenStorage( OLESTR("Image List"), NULL,
                                                STGM_READ | STGM_SHARE_EXCLUSIVE, 
                                                NULL, 0, &lpst ) == S_OK )
                           {
                              // Open the stream within the document.
                              //.....................................
                              lpst->lpVtbl->OpenStream( lpst, OLESTR("Images"), NULL, 
                                                  STGM_READ | STGM_SHARE_EXCLUSIVE,
						                                0, &lpstr );

                              // Read the image list from the document.
                              //.......................................
                              hList = ImageList_Read( lpstr );
                              InvalidateRect( hWnd, NULL, TRUE );

                              // Release the IStream and IStorage objects.
                              //..........................................
                              lpstr->lpVtbl->Release(lpstr);
                              lpst->lpVtbl->Release(lpst);
                           }
                        }
                        break;
                        

                 case IDM_ABOUT :
                        DialogBox( hInst, "AboutBox", hWnd, (DLGPROC)About );
                        break;

                 case IDM_EXIT :
                        DestroyWindow( hWnd );
                        break;
              }
              break;
      
      case WM_DESTROY :
              if ( hList )
                 ImageList_Destroy( hList );

              PostQuitMessage(0);
              break;

      default :
            return( DefWindowProc( hWnd, uMsg, wParam, lParam ) );
   }

   return( 0L );
}




LRESULT CALLBACK About( HWND hDlg,           
                        UINT message,        
                        WPARAM wParam,       
                        LPARAM lParam)
{
   switch (message) 
   {
       case WM_INITDIALOG: 
               return (TRUE);

       case WM_COMMAND:                              
               if (   LOWORD(wParam) == IDOK         
                   || LOWORD(wParam) == IDCANCEL)    
               {
                       EndDialog(hDlg, TRUE);        
                       return (TRUE);
               }
               break;
   }

   return (FALSE); 
}
